# flake8: noqa

# from src.silver_surfer_models.resources.PatentApplication import PatentApplicationModel
# from src.silver_surfer_models.resources.Trial.models import TrialModel
# from src.silver_surfer_models.resources.Patent.models import PatentModel
from src.silver_surfer_models.resources.File.models import FileModel
